# Generated by Django 5.1 on 2024-08-19 14:11

import django_db_views.migration_functions
import django_db_views.operations
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0016_missingscreeningogtt"),
        ("meta_screening", "0067_alter_historicalscreeningpartone_report_datetime_and_more"),
    ]

    operations = [
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, now() as created,\n    'meta_reports.missing_screening_ogtt_view' as report_model\n      from (\n        select screening_identifier, site_id, report_datetime as 'screening_datetime', fbg_datetime,\n        fbg_value,  ogtt_value, fbg2_value, ogtt2_value, fbg2_datetime, ogtt2_datetime, consented \n        from meta_screening_subjectscreening \n        where fbg_value is not null and ogtt_value is null\n    ) as A\n    order by screening_identifier",
                "missing_screening_ogtt_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "missing_screening_ogtt_view", engine="django.db.backends.mysql"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, get_random_uuid() as id, now() as created,\n    'meta_reports.missing_screening_ogtt_view' as report_model\n      from (\n        select screening_identifier, site_id, report_datetime as 'screening_datetime', fbg_datetime,\n        fbg_value,  ogtt_value, fbg2_value, ogtt2_value, fbg2_datetime, ogtt2_datetime, consented \n        from meta_screening_subjectscreening \n        where fbg_value is not null and ogtt_value is null\n    ) as A\n    order by screening_identifier",
                "missing_screening_ogtt_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "missing_screening_ogtt_view", engine="django.db.backends.postgresql"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "SELECT *, lower(\n    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n    substr(hex( randomblob(2)), 2) || '-' ||\n    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || '-' ||\n    hex(randomblob(6))\n  ) as id, datetime() as `created`,\n    'meta_reports.missing_screening_ogtt_view' as report_model\n      from (\n        select screening_identifier, site_id, report_datetime as 'screening_datetime', fbg_datetime,\n        fbg_value,  ogtt_value, fbg2_value, ogtt2_value, fbg2_datetime, ogtt2_datetime, consented \n        from meta_screening_subjectscreening \n        where fbg_value is not null and ogtt_value is null\n    ) as A\n    order by screening_identifier",
                "missing_screening_ogtt_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "missing_screening_ogtt_view", engine="django.db.backends.sqlite3"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "# noqa\n    select *, uuid() as id, now() as created, 'meta_reports.unattendedthreeinrow' as `report_model` from (\n        select subject_identifier, site_id, appt_datetime, `first_value`, `second_value`, `third_value`,\n            datediff(third_date, first_date) as `interval_days`,\n            datediff(now(), first_date) as `from_now_days`\n        from (\n            select subject_identifier, site_id, appt_datetime,\n            FIRST_VALUE(visit_code) OVER w as `first_value`,\n            NTH_VALUE(visit_code, 2) OVER w as `second_value`,\n            NTH_VALUE(visit_code, 3) OVER w as `third_value`,\n            FIRST_VALUE(appt_datetime) OVER w as `first_date`,\n            NTH_VALUE(appt_datetime, 3) OVER w as `third_date`\n            from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n            and appt_datetime <= now()\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n        ) as A\n    where `second_value` is not null and `third_value` is not null\n    ) as B\n    order by site_id, from_now_days desc",
                "unattended_three_in_row_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, uuid() as id, now() as created, 'meta_reports.unattendedthreeinrow' as `report_model` from (\n        select subject_identifier, site_id, appt_datetime, `first_value`, `second_value`, `third_value`,\n            datediff(third_date, first_date) as `interval_days`,\n            datediff(now(), first_date) as `from_now_days`\n        from (\n            select subject_identifier, site_id, appt_datetime,\n            FIRST_VALUE(visit_code) OVER w as `first_value`,\n            NTH_VALUE(visit_code, 2) OVER w as `second_value`,\n            NTH_VALUE(visit_code, 3) OVER w as `third_value`,\n            FIRST_VALUE(appt_datetime) OVER w as `first_date`,\n            NTH_VALUE(appt_datetime, 3) OVER w as `third_date`\n            from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n            and appt_datetime <= now()\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n        ) as A\n    where `second_value` is not null and `third_value` is not null\n    ) as B\n    order by site_id, from_now_days desc",
                "unattended_three_in_row_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "# noqa\n    select *, gen_random_uuid() as id, now() as created,\n    'meta_reports.unattendedthreeinrow' as report_model\n    from (\n    select subject_identifier, site_id, appt_datetime, first_value, second_value, third_value,\n    EXTRACT(DAY FROM third_date - first_date) as interval_days,\n    EXTRACT(DAY FROM now() - first_date) as from_now_days\n    from (\n    select subject_identifier,site_id,appt_datetime,\n    FIRST_VALUE(visit_code) OVER w as first_value,\n    NTH_VALUE(visit_code, 2) OVER w as second_value,\n    NTH_VALUE(visit_code, 3) OVER w as third_value,\n    FIRST_VALUE(appt_datetime) OVER w as first_date,\n    NTH_VALUE(appt_datetime, 3) OVER w as third_date\n    from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n    and appt_datetime <= now()\n    WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n    ) as A\n    where second_value is not null and third_value is not null\n    ) as B\n    order by site_id, from_now_days desc",
                "unattended_three_in_row_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, gen_random_uuid() as id, now() as created, \n    'meta_reports.unattendedthreeinrow' as report_model \n    from (\n    select subject_identifier, site_id, appt_datetime, first_value, second_value, third_value,\n    EXTRACT(DAY FROM third_date - first_date) as interval_days,\n    EXTRACT(DAY FROM now() - first_date) as from_now_days \n    from (\n    select subject_identifier,site_id,appt_datetime,\n    FIRST_VALUE(visit_code) OVER w as first_value,\n    NTH_VALUE(visit_code, 2) OVER w as second_value,\n    NTH_VALUE(visit_code, 3) OVER w as third_value,\n    FIRST_VALUE(appt_datetime) OVER w as first_date,\n    NTH_VALUE(appt_datetime, 3) OVER w as third_date\n    from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n    and appt_datetime <= now()\n    WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n    ) as A\n    where second_value is not null and third_value is not null\n    ) as B\n    order by site_id, from_now_days desc",
                "unattended_three_in_row_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "# noqa\nSELECT *, lower(\n    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n    substr(hex( randomblob(2)), 2) || '-' ||\n    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || '-' ||\n    hex(randomblob(6))\n  ) as id, datetime() as created,'meta_reports.unattendedthreeinrow' as report_model from (\n        select subject_identifier, site_id, appt_datetime, first_value, second_value, third_value,\n            CAST(JulianDay(third_date) - JulianDay(first_date) AS INTEGER) as interval_days,\n            CAST(JulianDay(datetime()) - JulianDay(first_date) AS INTEGER) as from_now_days\n        from (\n            select subject_identifier, site_id, appt_datetime,\n            FIRST_VALUE(visit_code) OVER w as first_value,\n            NTH_VALUE(visit_code, 2) OVER w as second_value,\n            NTH_VALUE(visit_code, 3) OVER w as third_value,\n            FIRST_VALUE(appt_datetime) OVER w as first_date,\n            NTH_VALUE(appt_datetime, 3) OVER w as third_date\n            from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n            and appt_datetime <= datetime()\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n        ) as A\n    where second_value is not null and third_value is not null\n    ) as B\n    order by site_id, from_now_days desc",
                "unattended_three_in_row_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "SELECT *, lower(\n    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n    substr(hex( randomblob(2)), 2) || '-' ||\n    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || '-' ||\n    hex(randomblob(6))\n  ) as id, datetime() as created,'meta_reports.unattendedthreeinrow' as report_model from (\n        select subject_identifier, site_id, appt_datetime, first_value, second_value, third_value,\n            CAST(JulianDay(third_date) - JulianDay(first_date) AS Integer) as interval_days,\n            CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer) as from_now_days\n        from (\n            select subject_identifier, site_id, appt_datetime,\n            FIRST_VALUE(visit_code) OVER w as first_value,\n            NTH_VALUE(visit_code, 2) OVER w as second_value,\n            NTH_VALUE(visit_code, 3) OVER w as third_value,\n            FIRST_VALUE(appt_datetime) OVER w as first_date,\n            NTH_VALUE(appt_datetime, 3) OVER w as third_date\n            from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n            and appt_datetime <= datetime()\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n        ) as A\n    where second_value is not null and third_value is not null\n    ) as B\n    order by site_id, from_now_days desc",
                "unattended_three_in_row_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "# noqa\n    with appointments as (\n        select subject_identifier, site_id, visit_code, visit_code_sequence, appt_datetime,\n        case when appt_timing='missed' then 'New' else appt_status end as `appt_status`,\n        case when appt_timing='missed' then 1 else 0 end as `missed`\n        from edc_appointment_appointment\n        where visit_code_sequence=0 and appt_datetime<=now()\n        order by subject_identifier, appt_datetime\n        )\n    select *, uuid() as `id`, now() as `created`, 'meta_reports.unattendedthreeinrow2' as `report_model` from (\n        select  distinct subject_identifier, site_id,  `first_value`, `second_value`, `third_value`,\n            datediff(third_date, first_date) as `interval_days`,\n            datediff(now(), first_date) as `from_now_days`,\n            `first_status`, `second_status`, `third_status`, sum(missed) as `missed_count`\n        from (\n            select subject_identifier,site_id,appt_datetime, missed,\n            FIRST_VALUE(appt_status) OVER w as `third_status`,\n            NTH_VALUE(appt_status, 2) OVER w as `second_status`,\n            NTH_VALUE(appt_status, 3) OVER w as `first_status`,\n            FIRST_VALUE(visit_code) OVER w as `third_value`,\n            NTH_VALUE(visit_code, 2) OVER w as `second_value`,\n            NTH_VALUE(visit_code, 3) OVER w as `first_value`,\n            FIRST_VALUE(appt_datetime) OVER w as `third_date`,\n            NTH_VALUE(appt_datetime, 2) OVER w as `second_date`,\n            NTH_VALUE(appt_datetime, 3) OVER w as `first_date`\n            from appointments\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)\n        ) as A\n        where `second_value` is not null and `third_value` is not null\n          and `first_status`='New'\n          and `second_status`='New'\n          and `third_status`='New'\n        group by subject_identifier, site_id,  `first_value`, `second_value`, `third_value`,\n                 datediff(`third_date`, `first_date`),\n                 datediff(now(), `first_date`),\n                 `first_status`, `second_status`, `third_status`\n        order by subject_identifier, site_id\n    ) as B",
                "unattended_three_in_row2_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "with appointments as (\n        select subject_identifier, site_id, visit_code, visit_code_sequence, appt_datetime,\n        case when appt_timing='missed' then 'New' else appt_status end as `appt_status`,\n        case when appt_timing='missed' then 1 else 0 end as `missed`\n        from edc_appointment_appointment\n        where visit_code_sequence=0 and appt_datetime<=now()\n        order by subject_identifier, appt_datetime\n        )\n    select *, uuid() as `id`, now() as `created`, 'meta_reports.unattendedthreeinrow2' as `report_model` from (\n        select  distinct subject_identifier, site_id,  `first_value`, `second_value`, `third_value`,\n            datediff(third_date, first_date) as `interval_days`,\n            datediff(now(), first_date) as `from_now_days`,\n            `first_status`, `second_status`, `third_status`, sum(missed) as `missed_count`\n        from (\n            select subject_identifier,site_id,appt_datetime, missed,\n            FIRST_VALUE(appt_status) OVER w as `third_status`,\n            NTH_VALUE(appt_status, 2) OVER w as `second_status`,\n            NTH_VALUE(appt_status, 3) OVER w as `first_status`,\n            FIRST_VALUE(visit_code) OVER w as `third_value`,\n            NTH_VALUE(visit_code, 2) OVER w as `second_value`,\n            NTH_VALUE(visit_code, 3) OVER w as `first_value`,\n            FIRST_VALUE(appt_datetime) OVER w as `third_date`,\n            NTH_VALUE(appt_datetime, 2) OVER w as `second_date`,\n            NTH_VALUE(appt_datetime, 3) OVER w as `first_date`\n            from appointments\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)\n        ) as A\n        where `second_value` is not null and `third_value` is not null\n          and `first_status`='New'\n          and `second_status`='New'\n          and `third_status`='New'\n        group by subject_identifier, site_id,  `first_value`, `second_value`, `third_value`,\n                 datediff(`third_date`, `first_date`),\n                 datediff(now(), `first_date`),\n                 `first_status`, `second_status`, `third_status`\n        order by subject_identifier, site_id\n    ) as B",
                "unattended_three_in_row2_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "# noqa\n    with appointments as (\n        select subject_identifier, site_id, visit_code, visit_code_sequence, appt_datetime,\n        case when appt_timing='missed' then 'New' else appt_status end as appt_status,\n        case when appt_timing='missed' then 1 else 0 end as missed\n        from edc_appointment_appointment\n        where visit_code_sequence=0 and appt_datetime<=now()\n        order by subject_identifier, appt_datetime\n        )\n    select *, gen_random_uuid() as id, now() as created, 'meta_reports.unattendedthreeinrow2' as report_model\n    from (\n        select  distinct subject_identifier, site_id,  first_value, second_value, third_value,\n            EXTRACT(DAY FROM third_date - first_date) as interval_days,\n            EXTRACT(DAY FROM now() - first_date) as from_now_days,\n            first_status, second_status, third_status, sum(missed) as missed_count\n        from (\n            select subject_identifier,site_id,appt_datetime, missed,\n            FIRST_VALUE(appt_status) OVER w as third_status,\n            NTH_VALUE(appt_status, 2) OVER w as second_status,\n            NTH_VALUE(appt_status, 3) OVER w as first_status,\n            FIRST_VALUE(visit_code) OVER w as third_value,\n            NTH_VALUE(visit_code, 2) OVER w as second_value,\n            NTH_VALUE(visit_code, 3) OVER w as first_value,\n            FIRST_VALUE(appt_datetime) OVER w as third_date,\n            NTH_VALUE(appt_datetime, 2) OVER w as second_date,\n            NTH_VALUE(appt_datetime, 3) OVER w as first_date\n            from appointments\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)\n        ) as A\n        where second_value is not null and third_value is not null\n          and first_status='New'\n          and second_status='New'\n          and third_status='New'\n        group by subject_identifier, site_id,  first_value, second_value, third_value,\n                 EXTRACT(DAY FROM third_date - first_date),\n                 EXTRACT(DAY FROM now() - first_date),\n                 first_status, second_status, third_status\n        order by subject_identifier, site_id\n    ) as B",
                "unattended_three_in_row2_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "with appointments as (\n        select subject_identifier, site_id, visit_code, visit_code_sequence, appt_datetime,\n        case when appt_timing='missed' then 'New' else appt_status end as appt_status,\n        case when appt_timing='missed' then 1 else 0 end as missed\n        from edc_appointment_appointment\n        where visit_code_sequence=0 and appt_datetime<=now()\n        order by subject_identifier, appt_datetime\n        )\n    select *, gen_random_uuid() as id, now() as created, 'meta_reports.unattendedthreeinrow2' as report_model \n    from (\n        select  distinct subject_identifier, site_id,  first_value, second_value, third_value,\n            EXTRACT(DAY FROM third_date - first_date) as interval_days,\n            EXTRACT(DAY FROM now() - first_date) as from_now_days,\n            first_status, second_status, third_status, sum(missed) as missed_count\n        from (\n            select subject_identifier,site_id,appt_datetime, missed,\n            FIRST_VALUE(appt_status) OVER w as third_status,\n            NTH_VALUE(appt_status, 2) OVER w as second_status,\n            NTH_VALUE(appt_status, 3) OVER w as first_status,\n            FIRST_VALUE(visit_code) OVER w as third_value,\n            NTH_VALUE(visit_code, 2) OVER w as second_value,\n            NTH_VALUE(visit_code, 3) OVER w as first_value,\n            FIRST_VALUE(appt_datetime) OVER w as third_date,\n            NTH_VALUE(appt_datetime, 2) OVER w as second_date,\n            NTH_VALUE(appt_datetime, 3) OVER w as first_date\n            from appointments\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)\n        ) as A\n        where second_value is not null and third_value is not null\n          and first_status='New'\n          and second_status='New'\n          and third_status='New'\n        group by subject_identifier, site_id,  first_value, second_value, third_value,\n                 EXTRACT(DAY FROM third_date - first_date),\n                 EXTRACT(DAY FROM now() - first_date),\n                 first_status, second_status, third_status\n        order by subject_identifier, site_id\n    ) as B",
                "unattended_three_in_row2_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "# noqa\n    with appointments as (\n        select subject_identifier, site_id, visit_code, visit_code_sequence, appt_datetime,\n        case when appt_timing='missed' then 'New' else appt_status end as `appt_status`,\n        case when appt_timing='missed' then 1 else 0 end as `missed`\n        from edc_appointment_appointment\n        where visit_code_sequence=0 and appt_datetime<=now()\n        order by subject_identifier, appt_datetime\n        )\n    SELECT *, lower(\n            hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n            substr(hex( randomblob(2)), 2) || '-' ||\n            substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n            substr(hex(randomblob(2)), 2) || '-' ||\n            hex(randomblob(6))\n          ) as id, datetime() as created, 'meta_reports.unattendedthreeinrow2' as report_model from (\n        select  distinct subject_identifier, site_id,  first_value, second_value, third_value,\n            CAST(JulianDay(third_date) - JulianDay(first_date) AS Integer) as interval_days,\n            CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer) as from_now_days,\n            first_status, second_status, third_status, sum(missed) as missed_count\n        from (\n            select subject_identifier,site_id,appt_datetime, missed,\n            FIRST_VALUE(appt_status) OVER w as third_status,\n            NTH_VALUE(appt_status, 2) OVER w as second_status,\n            NTH_VALUE(appt_status, 3) OVER w as first_status,\n            FIRST_VALUE(visit_code) OVER w as third_value,\n            NTH_VALUE(visit_code, 2) OVER w as second_value,\n            NTH_VALUE(visit_code, 3) OVER w as first_value,\n            FIRST_VALUE(appt_datetime) OVER w as third_date,\n            NTH_VALUE(appt_datetime, 2) OVER w as second_date,\n            NTH_VALUE(appt_datetime, 3) OVER w as first_date\n            from appointments\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)\n        ) as A\n        where second_value is not null and third_value is not null\n          and first_status='New'\n          and second_status='New'\n          and third_status='New'\n        group by subject_identifier, site_id,  first_value, second_value, third_value,\n                 CAST(JulianDay(third_date) - JulianDay(first_date) AS Integer),\n                 CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer),\n                 first_status, second_status, third_status\n        order by subject_identifier, site_id\n    ) as B",
                "unattended_three_in_row2_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "with appointments as (\n        select subject_identifier, site_id, visit_code, visit_code_sequence, appt_datetime,\n        case when appt_timing='missed' then 'New' else appt_status end as `appt_status`,\n        case when appt_timing='missed' then 1 else 0 end as `missed`\n        from edc_appointment_appointment\n        where visit_code_sequence=0 and appt_datetime<=now()\n        order by subject_identifier, appt_datetime\n        )\n    SELECT *, lower(\n            hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n            substr(hex( randomblob(2)), 2) || '-' ||\n            substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n            substr(hex(randomblob(2)), 2) || '-' ||\n            hex(randomblob(6))\n          ) as id, datetime() as created, 'meta_reports.unattendedthreeinrow2' as report_model from (\n        select  distinct subject_identifier, site_id,  first_value, second_value, third_value,\n            CAST(JulianDay(third_date) - JulianDay(first_date) AS Integer) as interval_days,\n            CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer) as from_now_days,\n            first_status, second_status, third_status, sum(missed) as missed_count\n        from (\n            select subject_identifier,site_id,appt_datetime, missed,\n            FIRST_VALUE(appt_status) OVER w as third_status,\n            NTH_VALUE(appt_status, 2) OVER w as second_status,\n            NTH_VALUE(appt_status, 3) OVER w as first_status,\n            FIRST_VALUE(visit_code) OVER w as third_value,\n            NTH_VALUE(visit_code, 2) OVER w as second_value,\n            NTH_VALUE(visit_code, 3) OVER w as first_value,\n            FIRST_VALUE(appt_datetime) OVER w as third_date,\n            NTH_VALUE(appt_datetime, 2) OVER w as second_date,\n            NTH_VALUE(appt_datetime, 3) OVER w as first_date\n            from appointments\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)\n        ) as A\n        where second_value is not null and third_value is not null\n          and first_status='New'\n          and second_status='New'\n          and third_status='New'\n        group by subject_identifier, site_id,  first_value, second_value, third_value,\n                 CAST(JulianDay(third_date) - JulianDay(first_date) AS Integer),\n                 CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer),\n                 first_status, second_status, third_status\n        order by subject_identifier, site_id\n    ) as B",
                "unattended_three_in_row2_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "# noqa\n    select *, uuid() as `id`, now() as `created`, 'meta_reports.unattendedtwoinrow' as report_model from (\n        select subject_identifier, site_id, appt_datetime, `first_value`, `second_value`,\n            datediff(`second_date`, `first_date`) as `interval_days`,\n            datediff(now(), `first_date`) as `from_now_days`\n            from (\n                select subject_identifier ,site_id, appt_datetime,\n                FIRST_VALUE(`visit_code`) OVER w as `first_value`,\n                NTH_VALUE(`visit_code`, 2) OVER w as `second_value`,\n                NTH_VALUE(`visit_code`, 3) OVER w as `third_value`,\n                FIRST_VALUE(`appt_datetime`) OVER w as `first_date`,\n                NTH_VALUE(`appt_datetime`, 2) OVER w as `second_date`\n                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n                and appt_datetime <= now()\n                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n            ) as A1\n        where `second_value` is not null and `third_value` is null\n    ) as A2\n    order by site_id, `from_now_days` desc",
                "unattended_two_in_row_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, uuid() as `id`, now() as `created`, 'meta_reports.unattendedtwoinrow' as report_model from (\n        select subject_identifier, site_id, appt_datetime, `first_value`, `second_value`,\n            datediff(`second_date`, `first_date`) as `interval_days`,\n            datediff(now(), `first_date`) as `from_now_days`\n            from (\n                select subject_identifier ,site_id, appt_datetime,\n                FIRST_VALUE(`visit_code`) OVER w as `first_value`,\n                NTH_VALUE(`visit_code`, 2) OVER w as `second_value`,\n                NTH_VALUE(`visit_code`, 3) OVER w as `third_value`,\n                FIRST_VALUE(`appt_datetime`) OVER w as `first_date`,\n                NTH_VALUE(`appt_datetime`, 2) OVER w as `second_date`\n                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n                and appt_datetime <= now()\n                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n            ) as A1\n        where `second_value` is not null and `third_value` is null\n    ) as A2\n    order by site_id, `from_now_days` desc",
                "unattended_two_in_row_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "# noqa\n    select *, gen_random_uuid() as id, now() as created, 'meta_reports.unattendedtwoinrow' as report_model\n    from (\n        select subject_identifier, site_id, appt_datetime, first_value, second_value,\n            EXTRACT(DAY FROM second_date - first_date) as interval_days,\n            EXTRACT(DAY FROM now() - first_date) as from_now_days\n            from (\n                select subject_identifier, site_id, appt_datetime,\n                FIRST_VALUE(visit_code) OVER w as first_value,\n                NTH_VALUE(visit_code, 2) OVER w as second_value,\n                NTH_VALUE(visit_code, 3) OVER w as third_value,\n                FIRST_VALUE(appt_datetime) OVER w as first_date,\n                NTH_VALUE(appt_datetime, 2) OVER w as second_date\n                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n                and appt_datetime <= now()\n                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n            ) as A1\n        where second_value is not null and third_value is null\n    ) as A2\n    order by site_id, from_now_days desc",
                "unattended_two_in_row_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, gen_random_uuid() as id, now() as created, 'meta_reports.unattendedtwoinrow' as report_model \n    from (\n        select subject_identifier, site_id, appt_datetime, first_value, second_value,\n            EXTRACT(DAY FROM second_date - first_date) as interval_days,\n            EXTRACT(DAY FROM now() - first_date) as from_now_days\n            from (\n                select subject_identifier, site_id, appt_datetime,\n                FIRST_VALUE(visit_code) OVER w as first_value,\n                NTH_VALUE(visit_code, 2) OVER w as second_value,\n                NTH_VALUE(visit_code, 3) OVER w as third_value,\n                FIRST_VALUE(appt_datetime) OVER w as first_date,\n                NTH_VALUE(appt_datetime, 2) OVER w as second_date\n                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n                and appt_datetime <= now()\n                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n            ) as A1\n        where second_value is not null and third_value is null\n    ) as A2\n    order by site_id, from_now_days desc",
                "unattended_two_in_row_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "# noqa\nSELECT *, lower(\n    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n    substr(hex( randomblob(2)), 2) || '-' ||\n    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || '-' ||\n    hex(randomblob(6))\n  ) as id, datetime() as created, 'meta_reports.unattendedtwoinrow' as report_model from (\n        select subject_identifier, site_id, appt_datetime, first_value, second_value,\n            CAST(JulianDay(second_date) - JulianDay(first_date) AS Integer) as interval_days,\n            CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer) as from_now_days\n            from (\n                select subject_identifier ,site_id, appt_datetime,\n                FIRST_VALUE(visit_code) OVER w as first_value,\n                NTH_VALUE(visit_code, 2) OVER w as second_value,\n                NTH_VALUE(visit_code, 3) OVER w as third_value,\n                FIRST_VALUE(appt_datetime) OVER w as first_date,\n                NTH_VALUE(appt_datetime, 2) OVER w as second_date\n                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n                and appt_datetime <= datetime()\n                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n            ) as A1\n        where second_value is not null and third_value is null\n    ) as A2\n    order by site_id, from_now_days desc",
                "unattended_two_in_row_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "SELECT *, lower(\n    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n    substr(hex( randomblob(2)), 2) || '-' ||\n    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || '-' ||\n    hex(randomblob(6))\n  ) as id, datetime() as created, 'meta_reports.unattendedtwoinrow' as report_model from (\n        select subject_identifier, site_id, appt_datetime, first_value, second_value,\n            CAST(JulianDay(second_date) - JulianDay(first_date) AS Integer) as interval_days,\n            CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer) as from_now_days\n            from (\n                select subject_identifier ,site_id, appt_datetime,\n                FIRST_VALUE(visit_code) OVER w as first_value,\n                NTH_VALUE(visit_code, 2) OVER w as second_value,\n                NTH_VALUE(visit_code, 3) OVER w as third_value,\n                FIRST_VALUE(appt_datetime) OVER w as first_date,\n                NTH_VALUE(appt_datetime, 2) OVER w as second_date\n                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n                and appt_datetime <= datetime()\n                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n            ) as A1\n        where second_value is not null and third_value is null\n    ) as A2\n    order by site_id, from_now_days desc",
                "unattended_two_in_row_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
    ]



from django import forms
from edc_form_validators import FormValidatorMixin
from edc_model_form.mixins import BaseModelFormMixin
from edc_offstudy.modelform_mixins import OffstudyNonCrfModelFormMixin
from edc_sites.forms import SiteModelFormMixin
from edc_transfer.form_validators import SubjectTransferFormValidator

from ..models import SubjectTransfer


class SubjectTransferForm(
    SiteModelFormMixin,
    OffstudyNonCrfModelFormMixin,
    BaseModelFormMixin,
    FormValidatorMixin,
    forms.ModelForm,
):
    form_validator_cls = SubjectTransferFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    class Meta:
        model = SubjectTransfer
        fields = "__all__"



from __future__ import annotations

from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_subject_admin
from ..forms import HivExitReviewForm
from ..models import HivExitReview
from .modeladmin import CrfModelAdminMixin


@admin.register(HivExitReview, site=meta_subject_admin)
class HivExitReviewAdmin(CrfModelAdminMixin, SimpleHistoryAdmin):
    autocomplete_fields = ("current_arv_regimen",)

    form = HivExitReviewForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_visit",
                    "report_datetime",
                    "available",
                    "not_available_reason",
                )
            },
        ),
        (
            "HIV and ARVs",
            {
                "fields": (
                    "viral_load",
                    "viral_load_date",
                    "cd4",
                    "cd4_date",
                    "current_arv_regimen",
                    "other_current_arv_regimen",
                    "current_arv_regimen_start_date",
                )
            },
        ),
        ("Comment", {"fields": ("comment",)}),
        crf_status_fieldset,
        audit_fieldset_tuple,
    )

    list_filter = ("available",)

    radio_fields = {  # noqa: RUF012
        "available": admin.VERTICAL,
    }



import random

import edc_model.models
from django.db import models
from edc_model.models import BaseUuidModel
from edc_sites.model_mixins import SiteModelMixin

random.seed(237548)


class LabelData(SiteModelMixin, BaseUuidModel):
    subject_identifier = models.CharField(max_length=15, default="")
    sid = models.IntegerField(null=True, help_text="Pharmacy reference")
    reference = models.CharField(
        max_length=10, default="", unique=True, help_text="Bottle reference"
    )
    site_name = models.CharField(max_length=15, default="")
    gender = models.CharField(max_length=5, default="")
    pills_per_bottle = models.IntegerField(null=True, default=128)

    printed_datetime = models.DateTimeField(null=True)
    printed = models.BooleanField(default=False)
    scanned = models.BooleanField(default=False)
    scanned_datetime = models.DateTimeField(null=True)

    received_datetime = models.DateTimeField(null=True)
    received = models.BooleanField(default=False, help_text="Received at site")
    dispensed = models.BooleanField(default=False, help_text="Dispensed to clinic")
    dispensed_datetime = models.DateTimeField(null=True)
    crf = models.BooleanField(default=False, help_text="Entered into subject's CRF")
    crf_datetime = models.DateTimeField(null=True)

    history = edc_model.models.HistoricalRecords()

    class Meta:
        verbose_name = "Label Data"
        verbose_name_plural = "Label Data"



from edc_adherence.model_mixins import MedicationAdherenceModelMixin
from edc_model.models import BaseUuidModel

from ..model_mixins import CrfModelMixin


class MedicationAdherence(MedicationAdherenceModelMixin, CrfModelMixin, BaseUuidModel):
    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Medication Adherence"
        verbose_name_plural = "Medication Adherence"



# Generated by Django 2.2.6 on 2019-11-07 00:42

import django.core.validators
import edc_screening.model_mixins
import edc_sites.models
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("meta_screening", "0010_auto_20191106_0828")]

    operations = [
        migrations.AlterModelManagers(
            name="subjectscreening",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                (
                    "objects",
                    edc_screening.model_mixins.ScreeningManager(),
                ),
            ],
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="converted_fasting_glucose",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                default="mmol/L",
                max_length=15,
                verbose_name="Fasting glucose units",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="converted_fasting_glucose",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                default="mmol/L",
                max_length=15,
                verbose_name="Fasting glucose units",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="converted_fasting_glucose",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                default="mmol/L",
                max_length=15,
                verbose_name="Fasting glucose units",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="converted_fasting_glucose",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                default="mmol/L",
                max_length=15,
                verbose_name="Fasting glucose units",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="converted_fasting_glucose",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                default="mmol/L",
                max_length=15,
                verbose_name="Fasting glucose units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(50),
                ],
                verbose_name="Fasting glucose levels",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(50),
                ],
                verbose_name="Fasting glucose levels",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(50),
                ],
                verbose_name="Fasting glucose levels",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(50),
                ],
                verbose_name="Fasting glucose levels",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(50),
                ],
                verbose_name="Fasting glucose levels",
            ),
        ),
    ]



from clinicedc_constants import OTHER, YES
from edc_qareports.sql_generator import CrfCase

from meta_visit_schedule.constants import MONTH36, MONTH48

qa_cases = [
    CrfCase(
        label="No HIV Diagnosis date",
        dbtable="meta_subject_patienthistory",
        label_lower="meta_subject.patienthistory",
        fld_name="hiv_diagnosis_date",
    ),
    CrfCase(
        label="No VL value or VL date",
        dbtable="meta_subject_patienthistory",
        label_lower="meta_subject.patienthistory",
        where="crf.viral_load is null or crf.viral_load_date is null",
    ),
    CrfCase(
        label="No CD4 value or CD4 date",
        dbtable="meta_subject_patienthistory",
        label_lower="meta_subject.patienthistory",
        where="crf.cd4 is null or crf.cd4_date is null",
    ),
    CrfCase(
        label="No current ARV start date",
        dbtable="meta_subject_patienthistory",
        label_lower="meta_subject.patienthistory",
        fld_name="current_arv_regimen_start_date",
    ),
    CrfCase(
        label="No current ARV start date but previous ARV is YES",
        dbtable="meta_subject_patienthistory",
        label_lower="meta_subject.patienthistory",
        where=(
            f"crf.has_previous_arv_regimen ='{YES}' and "
            "crf.current_arv_regimen_start_date is null"
        ),
    ),
    CrfCase(
        label="Other current ARV regimen missing",
        dbtable="meta_subject_patienthistory",
        label_lower="meta_subject.patienthistory",
        where=f"crf.other_current_arv_regimen is null and arvregimen.name='{OTHER}'",
        list_tables=[("current_arv_regimen_id", "meta_lists_arvregimens", "arvregimen")],
    ),
    CrfCase(
        label="FBG/OGTT: missing OGTT",
        dbtable="meta_subject_glucose",
        label_lower="meta_subject.glucose",
        where=(
            "(crf.fbg_value is not null and crf.ogtt_value is null) "
            f"and ogtt_performed='{YES}'"
        ),
    ),
    CrfCase(
        label="Weight",
        dbtable="meta_subject_followupvitals",
        label_lower="meta_subject.followupvitals",
        where="(crf.weight is null and v.visit_code_sequence==0)",
    ),
    CrfCase(
        label="Waist circumference 36/48m",
        dbtable="meta_subject_followupvitals",
        label_lower="meta_subject.followupvitals",
        where=(
            f"(crf.waist_circumference_measured=='{YES}' and crf.waist_circumference "
            "is null) or (crf.waist_circumference is null and v.visit_code in "
            f"['{MONTH36}','{MONTH48}'] and v.visit_code_sequence==0)"
        ),
    ),
]



from edc_adverse_event.pdf_reports import DeathPdfReport as BaseDeathPdfReport

from .meta_pdf_report_mixin import MetaCrfReportMixin


class DeathPdfReport(MetaCrfReportMixin, BaseDeathPdfReport):
    pass



# Generated by Django 5.2.7 on 2025-10-03 22:06

from django.db import migrations


def fix_report_models(apps, schema_editor):
    note_cls = apps.get_model("edc_qareports", "Note")
    note_cls.objects.filter(report_model="meta_reports.onstudy_missing_values_view").update(
        report_model="meta_reports.onstudymissingvalues"
    )
    note_cls.objects.filter(
        report_model="meta_reports.onstudy_missing_lab_values_view"
    ).update(report_model="meta_reports.onstudymissinglabvalues")


class Migration(migrations.Migration):
    dependencies = [
        ("meta_reports", "0061_auto_20251004_0043"),
    ]

    operations = [migrations.RunPython(fix_report_models)]



# Generated by Django 5.1 on 2024-08-15 23:39

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0014_auto_20240813_1517"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.AlterField(
            model_name="endpoints",
            name="site",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.DO_NOTHING, to="sites.site"
            ),
        ),
    ]



from django import forms
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin

from ..models import ConcomitantMedication


class ConcomitantMedicationForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = CrfFormValidator

    class Meta:
        model = ConcomitantMedication
        fields = "__all__"



import sys

from multisite import SiteID

from .defaults import *  # noqa

sys.stdout.write(f"Settings file {__file__}\n")

SITE_ID = SiteID(default=1)
EDC_SITES_UAT_DOMAIN = False
EDC_MODEL_ADMIN_CSS_THEME = "edc_indigo"
LIVE_SYSTEM = True
EDC_SITES_DOMAIN_SUFFIX = "meta4.clinicedc.org"
AUTO_CREATE_KEYS = False



# Generated by Django 5.2.7 on 2025-10-14 12:56

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "meta_lists",
            "0020_alter_abnormalfootappearanceobservations_extra_value_and_more",
        ),
        ("meta_subject", "0237_historicalhivexitreview_singleton_field_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalhivexitreview",
            name="available",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=25,
                verbose_name="Are HIV test result and treatment information available?",
            ),
        ),
        migrations.AddField(
            model_name="historicalhivexitreview",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="May be left blank.",
                verbose_name="Any other comment?",
            ),
        ),
        migrations.AddField(
            model_name="historicalhivexitreview",
            name="not_available_reason",
            field=models.TextField(
                blank=True,
                default="",
                verbose_name="If not available, please explain ...",
            ),
        ),
        migrations.AddField(
            model_name="hivexitreview",
            name="available",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=25,
                verbose_name="Are HIV test result and treatment information available?",
            ),
        ),
        migrations.AddField(
            model_name="hivexitreview",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="May be left blank.",
                verbose_name="Any other comment?",
            ),
        ),
        migrations.AddField(
            model_name="hivexitreview",
            name="not_available_reason",
            field=models.TextField(
                blank=True,
                default="",
                verbose_name="If not available, please explain ...",
            ),
        ),
        migrations.AlterField(
            model_name="hivexitreview",
            name="current_arv_regimen",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="last_arv_regimen",
                to="meta_lists.arvregimens",
                verbose_name="Which antiretroviral therapy regimen is the patient currently on?",
            ),
        ),
    ]



import os
import sys

from multisite import SiteID

from .defaults import *  # noqa

sys.stdout.write(f"Settings file {__file__}\n")

# clear cache after changing site, for example:
# from django.core.cache import cache
# cache.clear()
# TZ Sites:
# SITE_ID = SiteID(default=20)  # Amana
# SITE_ID = SiteID(default=10)  # Hindu Mandal
# SITE_ID = SiteID(default=40)  # Mwananyamala
# SITE_ID = SiteID(default=50)  # Mbagala
# SITE_ID = SiteID(default=60)  # Mnazi-Moja
SITE_ID = SiteID(default=30)  # Temeke

INDEX_PAGE = "http://localhost:8000"
EDC_SITES_UAT_DOMAIN = False
DEBUG = True
ALLOWED_HOSTS = [
    "localhost",
    "mnazi-moja.tz.meta4.clinicedc.org",
    "mbagala.tz.meta4.clinicedc.org",
    "mwananyamala.tz.meta4.clinicedc.org",
    "hindu-mandal.tz.meta4.clinicedc.org",
    "temeke.tz.meta4.clinicedc.org",
    "amana.tz.meta4.clinicedc.org",
]

SECURE_SSL_REDIRECT = False
EDC_MODEL_ADMIN_CSS_THEME = "edc_purple"

if os.path.exists(BASE_DIR) and not os.path.exists(KEY_PATH):  # noqa
    os.makedirs(KEY_PATH)  # noqa
    AUTO_CREATE_KEYS = True

# if debugging, run `export DJANGO_DEBUG=True` before running runserver
# CELERY_TASK_ALWAYS_EAGER = os.getenv("DJANGO_DEBUG", "False") == "True"
# CELERY_TASK_EAGER_PROPAGATES = CELERY_TASK_ALWAYS_EAGER

EDC_PDF_REPORTS_WATERMARK_WORD = "SAMPLE"
EDC_PDF_REPORTS_WATERMARK_FONT = ("Helvetica", 100)
EDC_PHARMACY_LABEL_WATERMARK_WORD = "DO NOT USE"



from clinicedc_constants import NO, OTHER, YES
from django import forms
from django.core.exceptions import ObjectDoesNotExist
from edc_crf.crf_form_validator import CrfFormValidator
from edc_form_validators import INVALID_ERROR
from edc_utils import formatted_date
from edc_utils.date import to_local

from meta_prn.models import DmReferral


class DmFollowupFormValidator(CrfFormValidator):
    def clean(self):
        # referral_date must be before report datetime
        if (
            self.report_datetime
            and self.cleaned_data.get("referral_date")
            and self.cleaned_data.get("referral_date") >= to_local(self.report_datetime).date()
        ):
            self.raise_validation_error(
                {"referral_date": "Invalid. Expected a date prior to the report date above."},
                INVALID_ERROR,
            )

        # try to match referral date to the referal form
        try:
            dm_referral = DmReferral.objects.get(subject_identifier=self.subject_identifier)
        except ObjectDoesNotExist:
            self.raise_validation_error(
                {"__all__": "Original Referral form not found."}, INVALID_ERROR
            )
        else:
            if dm_referral.referral_date != self.cleaned_data.get("referral_date"):
                referral_dt = formatted_date(dm_referral.referral_date)
                report_dt = formatted_date(to_local(dm_referral.report_datetime).date())
                self.raise_validation_error(
                    {
                        "referral_date": (
                            f"Invalid. Expected `{referral_dt}` "
                            f"based on the referral form submitted on {report_dt}."
                        )
                    },
                    INVALID_ERROR,
                )

        # Diabetes clinic attendance
        self.m2m_required_if(
            NO,
            field="attended",
            m2m_field="missed_referral_reasons",
        )

        self.m2m_other_specify(
            OTHER,
            m2m_field="missed_referral_reasons",
            field_other="other_missed_referral_reason",
        )

        self.required_if(
            YES,
            field="attended",
            field_required="facility_attended",
        )
        self.required_if(
            YES,
            field="attended",
            field_required="attended_date",
        )

        # attended_date cannot be future relative to report datetime
        if (
            self.report_datetime
            and self.cleaned_data.get("attended_date")
            and self.cleaned_data.get("attended_date") > to_local(self.report_datetime).date()
        ):
            self.raise_validation_error(
                {"attended_date": "Invalid. Cannot be after report date above"}, INVALID_ERROR
            )

        # referral_date must be before report datetime
        if (
            self.cleaned_data.get("attended_date")
            and self.cleaned_data.get("referral_date")
            and self.cleaned_data.get("attended_date") < self.cleaned_data.get("referral_date")
        ):
            self.raise_validation_error(
                {"attended_date": "Invalid. Cannot be before the referral date"},
                INVALID_ERROR,
            )

        self.m2m_required_if(
            YES,
            field="attended",
            m2m_field="healthcare_workers",
        )
        self.m2m_other_specify(
            OTHER,
            m2m_field="healthcare_workers",
            field_other="other_healthcare_workers",
        )

        self.applicable_if(
            YES,
            field="attended",
            field_applicable="investigations_performed",
        )

        # investigations
        self.m2m_required_if(
            YES,
            field="investigations_performed",
            m2m_field="investigations",
        )

        self.m2m_other_specify(
            OTHER,
            m2m_field="investigations",
            field_other="other_investigations",
        )
        self.m2m_required_if(
            NO,
            field="complications_checks_performed",
            m2m_field="complications_checks",
        )
        self.m2m_other_specify(
            OTHER,
            m2m_field="complications_checks",
            field_other="other_complications_checks",
        )

        # dm treatment
        self.applicable_if(
            YES,
            field="attended",
            field_applicable="treatment_prescribed",
        )
        self.m2m_required_if(
            YES,
            field="treatment_prescribed",
            m2m_field="dm_treatments",
        )
        self.applicable_if(
            YES,
            field="treatment_prescribed",
            field_applicable="on_dm_medications",
        )
        self.required_if(
            YES,
            field="on_dm_medications",
            field_required="dm_medications_init_date",
        )

        # dm_medications_init_date must be before report_datetime
        if (
            self.report_datetime
            and self.cleaned_data.get("dm_medications_init_date")
            and self.cleaned_data.get("dm_medications_init_date")
            > to_local(self.report_datetime).date()
        ):
            self.raise_validation_error(
                {"dm_medications_init_date": "Invalid. Cannot be after the report date"},
                INVALID_ERROR,
            )

        # dm_medications_init_date must be after referral_date
        if self.cleaned_data.get("dm_medications_init_date") and self.cleaned_data.get(
            "dm_medications_init_date"
        ) < self.cleaned_data.get("referral_date"):
            self.raise_validation_error(
                {"dm_medications_init_date": "Invalid. Cannot be before the referral date"},
                INVALID_ERROR,
            )

        # dm_medications_init_date must be on or after attended_date
        if self.cleaned_data.get("dm_medications_init_date") and self.cleaned_data.get(
            "dm_medications_init_date"
        ) < self.cleaned_data.get("attended_date"):
            self.raise_validation_error(
                {
                    "dm_medications_init_date": (
                        "Invalid. Expected a date on or after date patient "
                        "attended the health facility."
                    )
                },
                INVALID_ERROR,
            )

        self.m2m_required_if(
            YES,
            field="on_dm_medications",
            m2m_field="dm_medications",
        )
        self.m2m_other_specify(
            OTHER,
            m2m_field="dm_medications",
            field_other="other_dm_medications",
        )

        # Diabetes Medication Adherence
        self.applicable_if(
            YES,
            field="on_dm_medications",
            field_applicable="medications_adherent",
        )
        self.required_if(
            YES,
            field="on_dm_medications",
            field_required="last_missed_pill",
        )

        self.confirm_visual_scores_match()

    def confirm_visual_scores_match(self) -> None:
        self.required_if(
            YES,
            field="on_dm_medications",
            field_required="visual_score_slider",
            inverse=False,
        )
        self.required_if(
            YES,
            field="on_dm_medications",
            field_required="visual_score_confirmed",
        )
        if (
            self.cleaned_data.get("on_dm_medications")
            and self.cleaned_data.get("on_dm_medications") == YES
        ):
            confirmed = self.cleaned_data.get("visual_score_confirmed")
            if (
                confirmed is not None
                and int(self.cleaned_data.get("visual_score_slider", "0")) != confirmed
            ):
                raise forms.ValidationError(
                    {"visual_score_confirmed": "Does not match visual score above."}
                )
